# -*- coding: utf-8; -*-
################################################################################
#
#  Rattail -- Retail Software Framework
#  Copyright © 2010-2023 Lance Edgar
#
#  This file is part of Rattail.
#
#  Rattail is free software: you can redistribute it and/or modify it under the
#  terms of the GNU General Public License as published by the Free Software
#  Foundation, either version 3 of the License, or (at your option) any later
#  version.
#
#  Rattail is distributed in the hope that it will be useful, but WITHOUT ANY
#  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
#  details.
#
#  You should have received a copy of the GNU General Public License along with
#  Rattail.  If not, see <http://www.gnu.org/licenses/>.
#
################################################################################
"""
Views for CORE member batches
"""

from corepos.db.office_op import model as corepos

from rattail_corepos.db.model import CoreMemberBatch, CoreMemberBatchRow
from rattail_corepos.batch.coremember import CoreMemberBatchHandler

from tailbone.views.batch import BatchMasterView
from tailbone_corepos.db import CoreOfficeSession


class CoreMemberBatchView(BatchMasterView):
    """
    Master view for CORE member batches.
    """
    model_class = CoreMemberBatch
    model_row_class = CoreMemberBatchRow
    batch_handler_class = CoreMemberBatchHandler
    route_prefix = 'batch.coremember'
    url_prefix = '/batch/corepos-member'
    downloadable = True
    rows_bulk_deletable = True

    grid_columns = [
        'id',
        'description',
        'input_file',
        'created',
        'created_by',
        'rowcount',
        'executed',
        'executed_by',
    ]

    form_fields = [
        'id',
        'input_file',
        'description',
        'notes',
        'params',
        'created',
        'created_by',
        'rowcount',
        'executed',
        'executed_by',
    ]

    row_labels = {
        'email1': "Email",
        'member_type_id': "Member Type",
    }

    row_grid_columns = [
        'sequence',
        'card_number',
        'first_name',
        'last_name',
        'email1',
        'phone',
        'member_type_id',
        'status_code',
    ]

    row_form_fields = [
        'sequence',
        'card_number',
        'card_number_raw',
        'status_code',
        'status_text',
    ]

    def configure_grid(self, g):
        super(CoreMemberBatchView, self).configure_grid(g)

        g.set_link('input_file')

    def configure_form(self, f):
        super(CoreMemberBatchView, self).configure_form(f)

        # input_file
        if self.creating:
            f.set_type('input_file', 'file')
        else:
            f.set_readonly('input_file')
            f.set_renderer('input_file', self.render_downloadable_file)

    def configure_row_grid(self, g):
        super(CoreMemberBatchView, self).configure_row_grid(g)

        app = self.get_rattail_app()
        member_types = app.cache_model(CoreOfficeSession(),
                                       corepos.MemberType,
                                       normalizer=lambda mtype: mtype.description,
                                       key=lambda mtype, normal: mtype.id)
        g.set_enum('member_type_id', member_types)

        g.set_link('card_number')
        g.set_link('first_name')
        g.set_link('last_name')
        g.set_link('email1')

    def row_grid_extra_class(self, row, i):
        if row.status_code == row.STATUS_MEMBER_NOT_FOUND:
            return 'warning'
        if row.status_code in (row.STATUS_FIELDS_CHANGED,):
            return 'notice'

    def template_kwargs_view_row(self, **kwargs):
        app = self.get_rattail_app()
        batch = kwargs['parent_instance']
        row = kwargs['instance']
        kwargs['batch'] = batch
        kwargs['instance_title'] = batch.id_str

        fields = batch.get_param('fields')

        kwargs['diff_fields'] = fields
        kwargs['diff_old_values'] = dict([
            (field, getattr(row, '{}_old'.format(field)))
            for field in fields])
        kwargs['diff_new_values'] = dict([
            (field, getattr(row, field))
            for field in fields])

        # CORE Office URL
        kwargs['core_office_url'] = None
        if row.card_number:
            corepos = app.get_corepos_handler()
            office_url = corepos.get_office_url()
            if not office_url:
                kwargs['core_office_why_no_url'] = "CORE Office URL is not configured"
            else:
                url = corepos.get_office_member_url(row.card_number, office_url=office_url)
                if url:
                    kwargs['core_office_url'] = url
                else:
                    kwargs['core_office_why_no_url'] = "URL not defined for this object"

        else:
            kwargs['core_office_why_no_url'] = "Card number not valid"

        return kwargs


def includeme(config):
    CoreMemberBatchView.defaults(config)



# -*- coding: utf-8; -*-
################################################################################
#
#  Rattail -- Retail Software Framework
#  Copyright © 2010-2023 Lance Edgar
#
#  This file is part of Rattail.
#
#  Rattail is free software: you can redistribute it and/or modify it under the
#  terms of the GNU General Public License as published by the Free Software
#  Foundation, either version 3 of the License, or (at your option) any later
#  version.
#
#  Rattail is distributed in the hope that it will be useful, but WITHOUT ANY
#  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
#  details.
#
#  You should have received a copy of the GNU General Public License along with
#  Rattail.  If not, see <http://www.gnu.org/licenses/>.
#
################################################################################
"""
Customer Views
"""

from webhelpers2.html import tags

from tailbone.views import ViewSupplement


class CustomerViewSupplement(ViewSupplement):
    """
    Customer view supplement for CORE integration
    """
    route_prefix = 'customers'

    labels = {
        'corepos_account_id': "CORE-POS Account ID",
        'corepos_card_number': "CORE-POS Card Number",
    }

    def get_grid_query(self, query):
        model = self.model
        return query.outerjoin(model.CoreCustomer)

    def configure_grid(self, g):
        model = self.model
        g.set_filter('corepos_account_id', model.CoreCustomer.corepos_account_id)
        g.set_filter('corepos_card_number', model.CoreCustomer.corepos_card_number)

    def configure_form(self, f):
        if not self.master.creating:
            f.append('corepos_account_id')
            f.append('corepos_card_number')

    def get_version_child_classes(self):
        model = self.model
        return [model.CoreCustomer]

    def get_xref_buttons(self, customer):
        app = self.get_rattail_app()
        corepos = app.get_corepos_handler()
        url = corepos.get_office_member_url(customer.number)
        if url:
            return [{'url': url, 'text': "View in CORE Office"}]

    def get_xref_links(self, customer):
        if customer.corepos_card_number:
            url = self.request.route_url('corepos.members.view',
                                         card_number=customer.corepos_card_number)
            return [tags.link_to("View CORE-POS Member", url)]


class CustomerShopperViewSupplement(ViewSupplement):
    """
    CustomerShopper view supplement for CORE integration
    """
    route_prefix = 'customer_shoppers'

    labels = {
        'corepos_customer_id': "CORE-POS Customer ID",
    }

    def get_grid_query(self, query):
        model = self.model
        return query.outerjoin(model.CoreCustomerShopper)

    def configure_grid(self, g):
        model = self.model
        g.append('corepos_customer_id')
        g.set_filter('corepos_customer_id', model.CoreCustomerShopper.corepos_customer_id)
        g.set_sorter('corepos_customer_id', model.CoreCustomerShopper.corepos_customer_id)

    def configure_form(self, f):
        if not self.master.creating:
            f.append('corepos_customer_id')

    def get_version_child_classes(self):
        model = self.model
        return [model.CoreCustomerShopper]


def includeme(config):
    CustomerViewSupplement.defaults(config)
    CustomerShopperViewSupplement.defaults(config)



# -*- coding: utf-8; -*-
################################################################################
#
#  Rattail -- Retail Software Framework
#  Copyright © 2010-2022 Lance Edgar
#
#  This file is part of Rattail.
#
#  Rattail is free software: you can redistribute it and/or modify it under the
#  terms of the GNU General Public License as published by the Free Software
#  Foundation, either version 3 of the License, or (at your option) any later
#  version.
#
#  Rattail is distributed in the hope that it will be useful, but WITHOUT ANY
#  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
#  details.
#
#  You should have received a copy of the GNU General Public License along with
#  Rattail.  If not, see <http://www.gnu.org/licenses/>.
#
################################################################################
"""
CORE-POS origin views
"""

from corepos.db.office_op import model as corepos

from deform import widget as dfwidget
from webhelpers2.html import tags

from .master import CoreOfficeMasterView


class OriginCountryView(CoreOfficeMasterView):
    """
    Master view for Origin Countries
    """
    model_class = corepos.OriginCountry
    model_title = "CORE-POS Origin Country"
    model_title_plural = "CORE-POS Origin Countries"
    url_prefix = '/core-pos/origins/countries'
    route_prefix = 'corepos.origin_countries'

    def configure_form(self, f):
        super(OriginCountryView, self).configure_form(f)

        if self.creating:
            f.remove('id')
        else:
            f.set_readonly('id')


class OriginStateProvinceView(CoreOfficeMasterView):
    """
    Master view for Origin State/Province
    """
    model_class = corepos.OriginStateProv
    model_title = "CORE-POS Origin State/Province"
    model_title_plural = "CORE-POS Origin States/Provinces"
    url_prefix = '/core-pos/origins/stateprov'
    route_prefix = 'corepos.origin_stateprov'

    def configure_form(self, f):
        super(OriginStateProvinceView, self).configure_form(f)

        if self.creating:
            f.remove('id')
        else:
            f.set_readonly('id')


class OriginRegionView(CoreOfficeMasterView):
    """
    Master view for Origin Custom Regions
    """
    model_class = corepos.OriginCustomRegion
    model_title = "CORE-POS Origin Region"
    url_prefix = '/core-pos/origins/regions'
    route_prefix = 'corepos.origin_regions'

    def configure_form(self, f):
        super(OriginRegionView, self).configure_form(f)

        if self.creating:
            f.remove('id')
        else:
            f.set_readonly('id')


class OriginView(CoreOfficeMasterView):
    """
    Base class for origin views.
    """
    model_class = corepos.Origin
    model_title = "CORE-POS Origin"
    url_prefix = '/core-pos/origins'
    route_prefix = 'corepos.origins'

    labels = {
        'country_id': "Country ID",
        'state_prov_id': "State/Province ID",
        'state_prov': "State/Province",
        'custom_id': "Custom Region ID",
    }

    grid_columns = [
        'id',
        'name',
        'short_name',
        'local',
        'country',
        'state_prov',
        'custom_region',
    ]

    form_fields = [
        'id',
        'name',
        'short_name',
        'local',
        'country',
        'state_prov',
        'custom_region',
    ]

    def configure_grid(self, g):
        super(OriginView, self).configure_grid(g)

        g.set_sort_defaults('id')
        g.set_link('name')
        g.set_link('short_name')

    def configure_form(self, f):
        super(OriginView, self).configure_form(f)

        # id
        if self.creating:
            f.remove('id')
        else:
            f.set_readonly('id')

        # country
        if self.creating or self.editing:
            f.replace('country', 'country_id')
            f.set_label('country_id', "Country")
            countries = self.Session.query(corepos.OriginCountry)\
                                    .order_by(corepos.OriginCountry.name)\
                                    .all()
            values = [(c.id, str(c)) for c in countries]
            f.set_widget('country_id', dfwidget.SelectWidget(values=values))
        else:
            f.set_renderer('country', self.render_country)

        # state_prov
        if self.creating or self.editing:
            f.replace('state_prov', 'state_prov_id')
            f.set_label('state_prov_id', "State/Province")
            stateprovs = self.Session.query(corepos.OriginStateProv)\
                                     .order_by(corepos.OriginStateProv.name)\
                                     .all()
            values = [(sp.id, str(sp)) for sp in stateprovs]
            f.set_widget('state_prov_id', dfwidget.SelectWidget(values=values))
        else:
            f.set_renderer('state_prov', self.render_stateprov)

        # custom_region
        if self.creating or self.editing:
            f.replace('custom_region', 'custom_id')
            f.set_label('custom_id', "Custom Region")
            regions = self.Session.query(corepos.OriginCustomRegion)\
                                  .order_by(corepos.OriginCustomRegion.name)\
                                  .all()
            values = [(r.id, str(r)) for r in regions]
            f.set_widget('custom_id', dfwidget.SelectWidget(values=values))
        else:
            f.set_renderer('custom_region', self.render_region)

    def render_country(self, origin, field):
        country = origin.country
        if country:
            text = str(country)
            if self.request.has_perm('corepos.origin_countries.view'):
                url = self.request.route_url('corepos.origin_countries.view', id=country.id)
                return tags.link_to(text, url)
            return text

    def render_stateprov(self, origin, field):
        state_prov = origin.state_prov
        if state_prov:
            text = str(state_prov)
            if self.request.has_perm('corepos.origin_stateprov.view'):
                url = self.request.route_url('corepos.origin_stateprov.view', id=state_prov.id)
                return tags.link_to(text, url)
            return text

    def render_region(self, origin, field):
        region = origin.custom_region
        if region:
            text = str(region)
            if self.request.has_perm('corepos.origin_regions.view'):
                url = self.request.route_url('corepos.origin_regions.view', id=region.id)
                return tags.link_to(text, url)
            return text


def defaults(config, **kwargs):
    base = globals()

    OriginCountryView = kwargs.get('OriginCountryView', base['OriginCountryView'])
    OriginCountryView.defaults(config)

    OriginStateProvinceView = kwargs.get('OriginStateProvinceView', base['OriginStateProvinceView'])
    OriginStateProvinceView.defaults(config)

    OriginRegionView = kwargs.get('OriginRegionView', base['OriginRegionView'])
    OriginRegionView.defaults(config)

    OriginView = kwargs.get('OriginView', base['OriginView'])
    OriginView.defaults(config)


def includeme(config):
    defaults(config)



# -*- coding: utf-8; -*-
################################################################################
#
#  Rattail -- Retail Software Framework
#  Copyright © 2010-2023 Lance Edgar
#
#  This file is part of Rattail.
#
#  Rattail is free software: you can redistribute it and/or modify it under the
#  terms of the GNU General Public License as published by the Free Software
#  Foundation, either version 3 of the License, or (at your option) any later
#  version.
#
#  Rattail is distributed in the hope that it will be useful, but WITHOUT ANY
#  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
#  details.
#
#  You should have received a copy of the GNU General Public License along with
#  Rattail.  If not, see <http://www.gnu.org/licenses/>.
#
################################################################################
"""
Member Views
"""

from webhelpers2.html import tags

from tailbone.views import ViewSupplement


class MembershipTypeViewSupplement(ViewSupplement):
    """
    MembershipType view supplement for CORE integration
    """
    route_prefix = 'membership_types'

    def get_xref_buttons(self, memtype):
        app = self.get_rattail_app()
        corepos = app.get_corepos_handler()
        url = corepos.get_office_url()
        if url:
            url = f'{url}/mem/MemberTypeEditor.php'
            return [{'url': url, 'text': "View in CORE Office"}]


class MemberViewSupplement(ViewSupplement):
    """
    Member view supplement for CORE integration
    """
    route_prefix = 'members'

    labels = {
        'corepos_account_id': "CORE-POS Account ID",
        'corepos_card_number': "CORE-POS Card Number",
    }

    def get_grid_query(self, query):
        model = self.model
        return query.outerjoin(model.CoreMember)

    def configure_grid(self, g):
        model = self.model
        g.set_filter('corepos_account_id', model.CoreMember.corepos_account_id)
        g.set_filter('corepos_card_number', model.CoreMember.corepos_card_number)

    def configure_form(self, f):
        f.append('corepos_account_id')
        f.append('corepos_card_number')

    def get_version_child_classes(self):
        model = self.model
        return [model.CoreMember]

    def get_xref_buttons(self, member):
        if member.customer and member.customer.corepos_card_number:
            app = self.get_rattail_app()
            corepos = app.get_corepos_handler()
            url = corepos.get_office_member_url(member.customer.corepos_card_number)
            if url:
                return [{'url': url, 'text': "View in CORE Office"}]

    def get_xref_links(self, member):
        if member.customer and member.customer.corepos_card_number:
            url = self.request.route_url('corepos.members.view',
                                         card_number=member.customer.corepos_card_number)
            return [tags.link_to("View CORE-POS Member", url)]


class MemberEquityPaymentViewSupplement(ViewSupplement):
    """
    Member view supplement for CORE integration
    """
    route_prefix = 'member_equity_payments'

    labels = {
        'corepos_card_number': "CORE-POS Card Number",
        'corepos_transaction_number': "CORE-POS Transaction Number",
        'corepos_transaction_id': "CORE-POS Transaction ID",
        'corepos_department_number': "CORE-POS Department Number",
        'corepos_datetime': "CORE-POS Date/Time",
    }

    def get_grid_query(self, query):
        model = self.model
        return query.outerjoin(model.CoreMemberEquityPayment)

    def configure_grid(self, g):
        model = self.model

        g.set_filter('corepos_card_number', model.CoreMemberEquityPayment.corepos_card_number)
        g.set_sorter('corepos_card_number', model.CoreMemberEquityPayment.corepos_card_number)

        g.set_filter('corepos_transaction_number', model.CoreMemberEquityPayment.corepos_transaction_number)
        g.set_sorter('corepos_transaction_number', model.CoreMemberEquityPayment.corepos_transaction_number)

        g.set_filter('corepos_transaction_id', model.CoreMemberEquityPayment.corepos_transaction_id)
        g.set_sorter('corepos_transaction_id', model.CoreMemberEquityPayment.corepos_transaction_id)

        g.set_filter('corepos_department_number', model.CoreMemberEquityPayment.corepos_department_number)
        g.set_sorter('corepos_department_number', model.CoreMemberEquityPayment.corepos_department_number)

        g.append('corepos_transaction_number')
        g.set_label('corepos_transaction_number', "CORE-POS Trans. No.")
        if 'corepos_transaction_number' in g.filters:
            g.filters['corepos_transaction_number'].label = self.labels['corepos_transaction_number']
        g.set_link('corepos_transaction_number')

        # corepos_datetime
        g.append('corepos_datetime')
        g.set_type('corepos_datetime', 'datetime')
        g.set_filter('corepos_datetime', model.CoreMemberEquityPayment.corepos_datetime)
        g.set_sorter('corepos_datetime', model.CoreMemberEquityPayment.corepos_datetime)

    def configure_form(self, f):

        f.append('corepos_card_number')
        f.set_readonly('corepos_card_number')

        f.append('corepos_transaction_number')
        f.set_readonly('corepos_transaction_number')

        f.append('corepos_transaction_id')
        f.set_readonly('corepos_transaction_id')

        f.append('corepos_department_number')
        f.set_readonly('corepos_department_number')

        f.append('corepos_datetime')
        f.set_readonly('corepos_datetime')

    def get_version_child_classes(self):
        model = self.model
        return [model.CoreMemberEquityPayment]

    def get_xref_buttons(self, payment):
        if payment.corepos_transaction_number and payment.corepos_card_number:
            app = self.get_rattail_app()
            corepos = app.get_corepos_handler()
            url = corepos.get_office_url()
            if url:
                url = f'{url}/reports/Equity/EquityReport.php?memNum={payment.corepos_card_number}'
                return [{'url': url, 'text': "View in CORE Office"}]

    def get_xref_links(self, payment):
        app = self.get_rattail_app()
        if payment.corepos_transaction_number:
            url = self.request.route_url('corepos.stock_purchases.view',
                                         card_number=payment.corepos_card_number,
                                         datetime=app.localtime(payment.corepos_datetime, from_utc=True),
                                         transaction_number=payment.corepos_transaction_number,
                                         department_number=payment.corepos_department_number)
            return [tags.link_to("View CORE-POS Stock Purchase", url)]


def includeme(config):
    MembershipTypeViewSupplement.defaults(config)
    MemberViewSupplement.defaults(config)
    MemberEquityPaymentViewSupplement.defaults(config)



# -*- coding: utf-8; -*-
################################################################################
#
#  Rattail -- Retail Software Framework
#  Copyright © 2010-2024 Lance Edgar
#
#  This file is part of Rattail.
#
#  Rattail is free software: you can redistribute it and/or modify it under the
#  terms of the GNU General Public License as published by the Free Software
#  Foundation, either version 3 of the License, or (at your option) any later
#  version.
#
#  Rattail is distributed in the hope that it will be useful, but WITHOUT ANY
#  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
#  details.
#
#  You should have received a copy of the GNU General Public License along with
#  Rattail.  If not, see <http://www.gnu.org/licenses/>.
#
################################################################################
"""
Views for CORE equity import batches
"""

from corepos.db.office_op import model as corepos

from rattail_corepos.db.model import CoreEquityImportBatch, CoreEquityImportBatchRow
from rattail_corepos.batch.equityimport import CoreEquityImportBatchHandler

from webhelpers2.html import tags

from tailbone.views.batch import BatchMasterView
from tailbone_corepos.db import CoreOfficeSession


class CoreEquityImportBatchView(BatchMasterView):
    """
    Master view for CORE member batches.
    """
    model_class = CoreEquityImportBatch
    model_row_class = CoreEquityImportBatchRow
    batch_handler_class = CoreEquityImportBatchHandler
    route_prefix = 'batch.corepos.equity_import'
    url_prefix = '/batch/corepos-equity-import'
    rows_bulk_deletable = True
    rows_deletable_if_executed = True

    row_labels = {
        'member_type_id': "Member Type",
        'corepos_equity_total': "CORE-POS Equity Total",
    }

    row_grid_columns = [
        'sequence',
        'card_number',
        'first_name',
        'last_name',
        'member_type_id',
        'payment_amount',
        'department_number',
        'tender_code',
        'timestamp',
        'status_code',
    ]

    row_form_fields = [
        'sequence',
        'payment',
        'card_number',
        'first_name',
        'last_name',
        'member_type_id',
        'payment_amount',
        'department_number',
        'tender_code',
        'timestamp',
        'corepos_equity_total',
        'rattail_equity_total',
        'other_equity_total',
        'status_code',
    ]

    def configure_row_grid(self, g):
        super().configure_row_grid(g)
        app = self.get_rattail_app()
        model = self.model

        # card_number
        g.set_link('card_number')
        if 'card_number' in g.filters:
            g.filters['card_number'].default_active = True
            g.filters['card_number'].default_verb = 'equal'

        # *_name
        g.set_link('first_name')
        g.set_link('last_name')

        # member_type_id
        query = self.Session.query(model.MembershipType)\
                            .order_by(model.MembershipType.name)
        memtypes = app.cache_model(self.Session(), model.MembershipType, query=query,
                                   key='number', normalizer=lambda t: str(t))
        g.set_enum('member_type_id', memtypes)

        # payment_amount
        g.set_type('payment_amount', 'currency')

        # department_number
        g.set_label('department_number', "Department")
        if 'department_number' in g.filters:
            g.filters['department_number'].label = "Department Number"

    def row_grid_extra_class(self, row, i):
        if row.status_code in (row.STATUS_MEMBER_NOT_FOUND,
                               row.STATUS_MISSING_VALUES,
                               row.STATUS_ALREADY_IN_CORE):
            return 'warning'
        if row.status_code in (row.STATUS_NEEDS_ATTENTION,
                               row.STATUS_EQUITY_OVERPAID):
            return 'notice'

    def configure_row_form(self, f):
        super().configure_row_form(f)
        app = self.get_rattail_app()
        model = self.model

        f.set_renderer('payment', self.render_payment)

        query = self.Session.query(model.MembershipType)\
                            .order_by(model.MembershipType.name)
        memtypes = app.cache_model(self.Session(), model.MembershipType, query=query,
                                   key='number', normalizer=lambda t: str(t))
        f.set_enum('member_type_id', memtypes)

        f.set_type('payment_amount', 'currency')
        f.set_type('corepos_equity_total', 'currency')
        f.set_type('rattail_equity_total', 'currency')
        f.set_type('other_equity_total', 'currency')

    def render_payment(self, row, field):
        payment = getattr(row, field)
        if not payment:
            return

        text = str(payment)
        url = self.request.route_url('member_equity_payments.view', uuid=payment.uuid)
        return tags.link_to(text, url)


def includeme(config):
    CoreEquityImportBatchView.defaults(config)



# -*- coding: utf-8; -*-
################################################################################
#
#  Rattail -- Retail Software Framework
#  Copyright © 2010-2023 Lance Edgar
#
#  This file is part of Rattail.
#
#  Rattail is free software: you can redistribute it and/or modify it under the
#  terms of the GNU General Public License as published by the Free Software
#  Foundation, either version 3 of the License, or (at your option) any later
#  version.
#
#  Rattail is distributed in the hope that it will be useful, but WITHOUT ANY
#  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
#  details.
#
#  You should have received a copy of the GNU General Public License along with
#  Rattail.  If not, see <http://www.gnu.org/licenses/>.
#
################################################################################
"""
Vendor Catalog batch views for CORE-POS
"""

from corepos.db.office_op import model as corepos

from deform import widget as dfwidget

from tailbone.views.batch import vendorcatalog as base
from tailbone_corepos.db import CoreOfficeSession


class VendorCatalogView(base.VendorCatalogView):
    """
    Master view for vendor catalog batches.
    """

    def configure_form(self, f):
        super(VendorCatalogView, self).configure_form(f)
        model = self.model

        # replace stock 'vendor' field with id/name combo
        f.remove('vendor', 'vendor_uuid')
        f.insert_after('parser_key', 'vendor_id')
        f.insert_after('vendor_id', 'vendor_name')

        # vendor_id
        if self.creating:
            vendors = CoreOfficeSession.query(corepos.Vendor)\
                                       .order_by(corepos.Vendor.name)\
                                       .all()
            values = [(str(vendor.id), vendor.name)
                      for vendor in vendors]
            f.set_widget('vendor_id', dfwidget.SelectWidget(values=values))
            f.set_required('vendor_id')
            f.set_label('vendor_id', "Vendor")

        # vendor_name
        if self.creating:
            f.remove('vendor_name')

    def get_batch_kwargs(self, batch):
        kwargs = super(VendorCatalogView, self).get_batch_kwargs(batch)

        if 'vendor_name' not in kwargs and batch.vendor_id:
            vendor = CoreOfficeSession.get(corepos.Vendor, batch.vendor_id)
            if vendor:
                kwargs['vendor_name'] = vendor.name

        return kwargs


def defaults(config, **kwargs):
    kwargs.setdefault('VendorCatalogView', VendorCatalogView)
    base.defaults(config, **kwargs)


def includeme(config):
    defaults(config)



# -*- coding: utf-8; -*-
################################################################################
#
#  Rattail -- Retail Software Framework
#  Copyright © 2010-2022 Lance Edgar
#
#  This file is part of Rattail.
#
#  Rattail is free software: you can redistribute it and/or modify it under the
#  terms of the GNU General Public License as published by the Free Software
#  Foundation, either version 3 of the License, or (at your option) any later
#  version.
#
#  Rattail is distributed in the hope that it will be useful, but WITHOUT ANY
#  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
#  details.
#
#  You should have received a copy of the GNU General Public License along with
#  Rattail.  If not, see <http://www.gnu.org/licenses/>.
#
################################################################################
"""
CORE-POS Like Code views
"""

from corepos.db.office_op import model as corepos

from .master import CoreOfficeMasterView


class LikeCodeView(CoreOfficeMasterView):
    """
    Base class for like code views.
    """
    model_class = corepos.LikeCode
    model_title = "CORE-POS Like Code"
    url_prefix = '/core-pos/like-codes'
    route_prefix = 'corepos.like_codes'
    results_downloadable = True

    has_rows = True
    model_row_class = corepos.Product

    labels = {
        'id': "Like Code",
        'preferred_vendor_id': "Preferred Vendor ID",
    }

    grid_columns = [
        'id',
        'description',
        'strict',
        'organic',
        'preferred_vendor_id',
        'multi_vendor',
        'sort_retail',
        'sort_internal',
    ]

    form_fields = [
        'id',
        'description',
        'strict',
        'organic',
        'preferred_vendor_id',
        'multi_vendor',
        'sort_retail',
        'sort_internal',
    ]

    row_labels = {
        'upc': "UPC",
    }

    row_grid_columns = [
        'upc',
        'brand',
        'description',
        'size',
        'department',
        'vendor',
        'normal_price',
        'scale',
        'cost',
    ]

    def configure_grid(self, g):
        super(LikeCodeView, self).configure_grid(g)

        g.remove_filter('likeCode')
        g.set_filter('id', corepos.LikeCode.id,
                     default_active=True,
                     default_verb='equal')

        g.sorters.pop('likeCode')
        g.set_sorter('id', corepos.LikeCode.id)
        g.set_sort_defaults('id')

        g.set_link('id')
        g.set_link('description')

    def get_row_data(self, likecode):
        return self.Session.query(corepos.Product)\
                           .join(corepos.ProductLikeCode)\
                           .filter(corepos.ProductLikeCode.like_code == likecode)

    def get_parent(self, product):
        return product._like_code.like_code

    def get_row_action_url(self, action, product, **kwargs):
        route_name = 'corepos.products.{}'.format(action)
        return self.request.route_url(route_name, id=product.id)

    def configure_row_grid(self, g):
        super(LikeCodeView, self).configure_row_grid(g)

        g.set_type('normal_price', 'currency')
        g.set_type('cost', 'currency')

        g.set_sort_defaults('upc')

        g.set_link('upc')
        g.set_link('description')


def defaults(config, **kwargs):
    base = globals()

    LikeCodeView = kwargs.get('LikeCodeView', base['LikeCodeView'])
    LikeCodeView.defaults(config)


def includeme(config):
    defaults(config)



# -*- coding: utf-8; -*-
################################################################################
#
#  Rattail -- Retail Software Framework
#  Copyright © 2010-2023 Lance Edgar
#
#  This file is part of Rattail.
#
#  Rattail is free software: you can redistribute it and/or modify it under the
#  terms of the GNU General Public License as published by the Free Software
#  Foundation, either version 3 of the License, or (at your option) any later
#  version.
#
#  Rattail is distributed in the hope that it will be useful, but WITHOUT ANY
#  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
#  details.
#
#  You should have received a copy of the GNU General Public License along with
#  Rattail.  If not, see <http://www.gnu.org/licenses/>.
#
################################################################################
"""
Employee Views
"""

from tailbone.views import ViewSupplement


class EmployeeViewSupplement(ViewSupplement):
    """
    Employee view supplement for CORE integration
    """
    route_prefix = 'employees'

    labels = {
        'corepos_number': "CORE-POS Number",
    }

    def get_grid_query(self, query):
        model = self.model
        return query.outerjoin(model.CoreEmployee)

    def configure_grid(self, g):
        model = self.model
        g.set_filter('corepos_number', model.CoreEmployee.corepos_number)

    def configure_form(self, f):
        f.append('corepos_number')

    def get_version_child_classes(self):
        model = self.model
        return [model.CoreEmployee]

    def get_xref_buttons(self, employee):
        app = self.get_rattail_app()
        url = app.get_corepos_handler().get_office_employee_url(employee.corepos_number)
        if url:
            return [{'url': url, 'text': "View in CORE Office"}]


def includeme(config):
    EmployeeViewSupplement.defaults(config)



# -*- coding: utf-8; -*-
################################################################################
#
#  Rattail -- Retail Software Framework
#  Copyright © 2010-2022 Lance Edgar
#
#  This file is part of Rattail.
#
#  Rattail is free software: you can redistribute it and/or modify it under the
#  terms of the GNU General Public License as published by the Free Software
#  Foundation, either version 3 of the License, or (at your option) any later
#  version.
#
#  Rattail is distributed in the hope that it will be useful, but WITHOUT ANY
#  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
#  details.
#
#  You should have received a copy of the GNU General Public License along with
#  Rattail.  If not, see <http://www.gnu.org/licenses/>.
#
################################################################################
"""
CORE-POS user views
"""

from corepos.db.office_op import model as corepos

from .master import CoreOfficeMasterView


class UserView(CoreOfficeMasterView):
    """
    Master view for CORE Office Users
    """
    model_class = corepos.User
    model_key = 'name'
    model_title = "CORE-POS User"
    url_prefix = '/core-pos/users'
    route_prefix = 'corepos.users'

    labels = {
        'uid': "UID",
        'name': "Username",
        'session_id': "Session ID",
        'totp_url': "TOTP URL",
    }

    grid_columns = [
        'uid',
        'name',
        'real_name',
        'email',
    ]

    form_fields = [
        'uid',
        'name',
        'real_name',
        'email',
        'session_id',
        'totp_url',
    ]

    def configure_grid(self, g):
        super(UserView, self).configure_grid(g)

        if not hasattr(corepos.User, 'email'):
            g.remove('email')

        g.set_sort_defaults('name')

        g.set_link('uid')
        g.set_link('name')
        g.set_link('real_name')
        g.set_link('email')

    def configure_form(self, f):
        super(UserView, self).configure_form(f)

        if not hasattr(corepos.User, 'email'):
            f.remove('email')


def defaults(config, **kwargs):
    base = globals()

    UserView = kwargs.get('UserView', base['UserView'])
    UserView.defaults(config)


def includeme(config):
    defaults(config)



# -*- coding: utf-8; -*-
################################################################################
#
#  Rattail -- Retail Software Framework
#  Copyright © 2010-2022 Lance Edgar
#
#  This file is part of Rattail.
#
#  Rattail is free software: you can redistribute it and/or modify it under the
#  terms of the GNU General Public License as published by the Free Software
#  Foundation, either version 3 of the License, or (at your option) any later
#  version.
#
#  Rattail is distributed in the hope that it will be useful, but WITHOUT ANY
#  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
#  details.
#
#  You should have received a copy of the GNU General Public License along with
#  Rattail.  If not, see <http://www.gnu.org/licenses/>.
#
################################################################################
"""
CORE-POS vendor views
"""

from corepos.db.office_op import model as corepos

from .master import CoreOfficeMasterView


class VendorView(CoreOfficeMasterView):
    """
    Base class for vendor views.
    """
    model_class = corepos.Vendor
    model_key = 'id'
    model_title = "CORE-POS Vendor"
    url_prefix = '/core-pos/vendors'
    route_prefix = 'corepos.vendors'
    creatable = True
    editable = True
    deletable = True

    labels = {
        'id': "ID",
    }

    grid_columns = [
        'id',
        'name',
        'abbreviation',
        'discount_rate',
        'contact',
    ]

    form_fields = [
        'id',
        'name',
        'abbreviation',
        'discount_rate',
        'phone',
        'fax',
        'email',
        'website',
        'notes',
    ]

    has_rows = True
    model_row_class = corepos.VendorItem

    row_labels = {
        'vendor_item_id': "Vendor Item ID",
        'sku': "SKU",
        'upc': "UPC",
    }

    row_grid_columns = [
        'vendor_item_id',
        'sku',
        'upc',
        'brand',
        'description',
        'size',
        'cost',
        'units',
        'modified',
    ]

    def configure_grid(self, g):
        super(VendorView, self).configure_grid(g)

        # TODO: this is only needed b/c of orm.synonym usage
        g.set_sorter('id', corepos.Vendor.id)

        g.set_link('id')
        g.set_link('name')
        g.set_link('abbreviation')

    # TODO: ugh, would be nice to not have to do this...
    def get_action_route_kwargs(self, row):
        return {'id': row.id}

    def configure_form(self, f):
        super(VendorView, self).configure_form(f)

        if self.creating:
            f.remove_field('contact')

    def core_office_object_url(self, office_url, vendor):
        return '{}/item/vendors/VendorIndexPage.php?vid={}'.format(
            office_url, vendor.id)

    def get_row_data(self, vendor):
        return self.Session.query(corepos.VendorItem)\
                           .filter(corepos.VendorItem.vendor == vendor)

    def get_parent(self, item):
        return item.vendor

    def configure_row_grid(self, g):
        super(VendorView, self).configure_row_grid(g)

        g.set_type('units', 'quantity')
        g.set_sort_defaults('sku')

    def row_view_action_url(self, item, i):
        return self.request.route_url('corepos.vendor_items.view',
                                      **{'vendor_id': item.vendor.id,
                                         'sku': item.sku})


class VendorItemView(CoreOfficeMasterView):
    """
    Base class for vendor iem views.
    """
    model_class = corepos.VendorItem
    model_title = "CORE-POS Vendor Item"
    url_prefix = '/core-pos/vendor-items'
    route_prefix = 'corepos.vendor_items'

    labels = {
        'vendor_item_id': "ID",
        'sku': "SKU",
        'vendor_id': "Vendor ID",
        'upc': "UPC",
        'vendor_department_id': "Vendor Department ID",
        'srp': "SRP",
    }

    grid_columns = [
        'vendor_item_id',
        'sku',
        'vendor',
        'upc',
        'brand',
        'description',
        'size',
        'cost',
        'units',
        'modified',
    ]

    form_fields = [
        'vendor_item_id',
        'sku',
        'vendor_id',
        'vendor',
        'upc',
        'brand',
        'description',
        'size',
        'units',
        'cost',
        'sale_cost',
        'vendor_department_id',
        'srp',
        'modified',
    ]

    def configure_grid(self, g):
        super(VendorItemView, self).configure_grid(g)

        g.filters['upc'].default_active = True
        g.filters['upc'].default_verb = 'contains'

        g.set_type('units', 'quantity')

        g.set_sort_defaults('modified', 'desc')

        g.set_link('vendor_item_id')
        g.set_link('sku')
        g.set_link('vendor')
        g.set_link('upc')
        g.set_link('brand')
        g.set_link('description')

    def configure_form(self, f):
        super(VendorItemView, self).configure_form(f)

        f.set_type('units', 'quantity')
        f.set_type('srp', 'currency')

        f.set_readonly('vendor')

        if self.creating:
            f.remove('vendor_item_id')
        else:
            f.set_readonly('vendor_item_id')

        if self.creating or self.editing:
            f.remove('modified')
        else:
            f.set_readonly('modified')


def defaults(config, **kwargs):
    base = globals()

    VendorView = kwargs.get('VendorView', base['VendorView'])
    VendorView.defaults(config)

    VendorItemView = kwargs.get('VendorItemView', base['VendorItemView'])
    VendorItemView.defaults(config)


def includeme(config):
    defaults(config)



# -*- coding: utf-8; -*-
################################################################################
#
#  Rattail -- Retail Software Framework
#  Copyright © 2010-2022 Lance Edgar
#
#  This file is part of Rattail.
#
#  Rattail is free software: you can redistribute it and/or modify it under the
#  terms of the GNU General Public License as published by the Free Software
#  Foundation, either version 3 of the License, or (at your option) any later
#  version.
#
#  Rattail is distributed in the hope that it will be useful, but WITHOUT ANY
#  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
#  details.
#
#  You should have received a copy of the GNU General Public License along with
#  Rattail.  If not, see <http://www.gnu.org/licenses/>.
#
################################################################################
"""
CORE-POS TableSyncRule views
"""

from corepos.db.office_op import model as corepos

from .master import CoreOfficeMasterView


class TableSyncRuleView(CoreOfficeMasterView):
    """
    Master view for table sync rules
    """
    model_class = corepos.TableSyncRule
    model_title = "CORE-POS Table Sync Rule"
    url_prefix = '/core-pos/table-sync-rules'
    route_prefix = 'corepos.table_sync_rules'


def defaults(config, **kwargs):
    base = globals()

    TableSyncRuleView = kwargs.get('TableSyncRuleView', base['TableSyncRuleView'])
    TableSyncRuleView.defaults(config)


def includeme(config):
    defaults(config)



# -*- coding: utf-8; -*-
################################################################################
#
#  Rattail -- Retail Software Framework
#  Copyright © 2010-2022 Lance Edgar
#
#  This file is part of Rattail.
#
#  Rattail is free software: you can redistribute it and/or modify it under the
#  terms of the GNU General Public License as published by the Free Software
#  Foundation, either version 3 of the License, or (at your option) any later
#  version.
#
#  Rattail is distributed in the hope that it will be useful, but WITHOUT ANY
#  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
#  details.
#
#  You should have received a copy of the GNU General Public License along with
#  Rattail.  If not, see <http://www.gnu.org/licenses/>.
#
################################################################################
"""
CORE-POS supersuperdepartment views
"""

from corepos.db.office_op import model as corepos

from .master import CoreOfficeMasterView
from . import departments as base


class SuperDepartmentView(base.DepartmentView):
    """
    Base class for super department views.  Which is really just a particular
    kind of view for certain "departments".
    """
    model_title = "CORE-POS Super Department"
    url_prefix = '/core-pos/super-departments'
    route_prefix = 'corepos.super_departments'
    creatable = False
    deletable = False

    has_rows = True
    model_row_class = corepos.Department
    rows_viewable = False

    grid_columns = [
        'number',
        'name',
        'members',
    ]

    form_fields = [
        'number',
        'name',
    ]

    row_labels = {
        'see_id': "See ID",
        'modified_by_id': "Modified by ID",
    }

    row_grid_columns = [
        'number',
        'name',
        'tax',
        'food_stampable',
        'limit',
        'minimum',
        'discount',
        'see_id',
        'modified',
        'modified_by_id',
        'margin',
        'sales_code',
        'member_only',
    ]

    def query(self, session):

        # find the distinct set of parent (super) department numbers
        query = session.query(corepos.SuperDepartment.parent_id).distinct()
        parent_numbers = [row[0] for row in query]

        # build the base/default Department query
        query = super(SuperDepartmentView, self).query(session)

        # if we actually have super departments, then we filter the query
        if parent_numbers:
            query = query.filter(corepos.Department.number.in_(parent_numbers))
        else: # otherwise we ensure query has no results
            query = query.filter(corepos.Department.number == None)

        return query

    def configure_grid(self, g):
        super(SuperDepartmentView, self).configure_grid(g)

        g.set_renderer('members', self.render_members)

        g.set_sort_defaults('name')

    def configure_form(self, f):
        super(SuperDepartmentView, self).configure_form(f)

        f.set_readonly('number')

    def render_members(self, department, field):
        return str(len(department._super_children))

    def get_row_data(self, parent):

        # find all "child" department numbers
        query = self.Session().query(corepos.SuperDepartment)\
                              .filter(corepos.SuperDepartment.parent == parent)
        child_numbers = [superdept.child_id for superdept in query]

        # return a query on Department with only those child numbers
        query = self.Session().query(corepos.Department)
        if child_numbers:
            query = query.filter(corepos.Department.number.in_(child_numbers))
        else:
            query = query.filter(corepos.Department.number == None)
        return query

    def get_parent(self, child):
        return child._super_parents[0].parent

    def configure_row_grid(self, g):
        super(SuperDepartmentView, self).configure_row_grid(g)

        g.set_type('modified', 'datetime_local')

        g.set_sort_defaults('number')


def defaults(config, **kwargs):
    base = globals()

    SuperDepartmentView = kwargs.get('SuperDepartmentView', base['SuperDepartmentView'])
    SuperDepartmentView.defaults(config)


def includeme(config):
    defaults(config)



# -*- coding: utf-8; -*-
################################################################################
#
#  Rattail -- Retail Software Framework
#  Copyright © 2010-2023 Lance Edgar
#
#  This file is part of Rattail.
#
#  Rattail is free software: you can redistribute it and/or modify it under the
#  terms of the GNU General Public License as published by the Free Software
#  Foundation, either version 3 of the License, or (at your option) any later
#  version.
#
#  Rattail is distributed in the hope that it will be useful, but WITHOUT ANY
#  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
#  details.
#
#  You should have received a copy of the GNU General Public License along with
#  Rattail.  If not, see <http://www.gnu.org/licenses/>.
#
################################################################################
"""
CORE-POS "archive" transaction views
"""

from collections import OrderedDict

from corepos.db.office_arch.model import TransactionDetail

from .base import TransactionDetailMasterView
from tailbone_corepos.db import CoreTransArchiveSession, ExtraCoreTransArchiveSessions


class TransactionDetailView(TransactionDetailMasterView):
    """
    Master view for "archive" transaction details.
    """
    model_class = TransactionDetail
    model_title = "CORE-POS Archived Transaction Detail"
    url_prefix = '/corepos/transaction-details/archive'
    route_prefix = 'corepos.transaction_details_archive'

    @property
    def Session(self):
        """
        Which session we return will depend on user's "current" engine.
        """
        dbkey = self.get_current_engine_dbkey()

        if dbkey != 'default' and dbkey in ExtraCoreTransArchiveSessions:
            return ExtraCoreTransArchiveSessions[dbkey]

        return CoreTransArchiveSession

    def get_db_engines(self):
        engines = OrderedDict()
        if self.rattail_config.core_office_trans_archive_engine:
            engines['default'] = self.rattail_config.core_office_trans_archive_engine
        for dbkey in self.rattail_config.core_office_trans_archive_engines:
            if dbkey != 'default':
                engines[dbkey] = self.rattail_config.core_office_trans_archive_engines[dbkey]
        return engines

    def make_isolated_session(self):
        app = self.get_rattail_app()
        corepos = app.get_corepos_handler()

        dbkey = self.get_current_engine_dbkey()
        engine = self.rattail_config.core_office_arch_engines.get(dbkey)
        if not engine:
            engine = self.rattail_config.core_office_arch_engine
        assert engine

        return corepos.make_session_office_arch(bind=engine)


def defaults(config, **kwargs):
    base = globals()

    TransactionDetailView = kwargs.get('TransactionDetailView', base['TransactionDetailView'])
    TransactionDetailView.defaults(config)


def includeme(config):
    defaults(config)



# -*- coding: utf-8; -*-
################################################################################
#
#  Rattail -- Retail Software Framework
#  Copyright © 2010-2022 Lance Edgar
#
#  This file is part of Rattail.
#
#  Rattail is free software: you can redistribute it and/or modify it under the
#  terms of the GNU General Public License as published by the Free Software
#  Foundation, either version 3 of the License, or (at your option) any later
#  version.
#
#  Rattail is distributed in the hope that it will be useful, but WITHOUT ANY
#  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
#  details.
#
#  You should have received a copy of the GNU General Public License along with
#  Rattail.  If not, see <http://www.gnu.org/licenses/>.
#
################################################################################
"""
CORE POS purchase order views
"""

from corepos.db.office_op import model as corepos

from .master import CoreOfficeMasterView


class PurchaseOrderView(CoreOfficeMasterView):
    """
    Master view for purchase orders
    """
    model_class = corepos.PurchaseOrder
    model_title = "CORE-POS Purchase Order"
    url_prefix = '/core-pos/purchase-orders'
    route_prefix = 'corepos.purchase_orders'

    has_rows = True
    model_row_class = corepos.PurchaseOrderItem
    rows_viewable = False
    rows_downloadable_xlsx = True

    labels = {
        'vendor_id': "Vendor ID",
        'store_id': "Store ID",
        'user_id': "User ID",
        'vendor_order_id': "Vendor Order ID",
        'vendor_invoice_id': "Vendor Invoice ID",
        'standing_id': "Standing ID",
        'transfer_id': "Transfer ID",
    }

    grid_columns = [
        'id',
        'vendor',
        'store_id',
        'creation_date',
        'placed',
        'placed_date',
        'vendor_order_id',
        'vendor_invoice_id',
    ]

    form_fields = [
        'id',
        'vendor_id',
        'vendor',
        'store_id',
        'store',
        'creation_date',
        'placed',
        'placed_date',
        'user_id',
        'vendor_order_id',
        'vendor_invoice_id',
        'standing_id',
        'inventory_ignore',
        'transfer_id',
        'notes',
    ]

    row_labels = {
        'sku': "SKU",
        'internal_upc': "Internal UPC",
        'is_special_order': "Special Order",
    }

    row_grid_columns = [
        'sku',
        'internal_upc',
        'brand',
        'description',
        'unit_size',
        'unit_cost',
        'case_size',
        'quantity',
        'received_date',
        'received_quantity',
        'received_total_cost',
        'is_special_order',
    ]

    def configure_grid(self, g):
        super(PurchaseOrderView, self).configure_grid(g)

        g.set_joiner('vendor', lambda q: q.outerjoin(
            corepos.Vendor,
            corepos.Vendor.id == corepos.PurchaseOrder.vendor_id))
        g.set_sorter('vendor', corepos.Vendor.name)
        g.set_filter('vendor', corepos.Vendor.name)

        # g.set_joiner('store', lambda q: q.outerjoin(
        #     corepos.Store,
        #     corepos.Store.id == corepos.PurchaseOrder.store_id))
        # g.set_sorter('store', corepos.Store.description)

        g.set_sorter('id', corepos.PurchaseOrder.id)
        g.set_sort_defaults('id', 'desc')

        g.set_link('id')
        g.set_link('vendor')

    def configure_form(self, f):
        super(PurchaseOrderView, self).configure_form(f)

        f.set_renderer('vendor', self.render_corepos_vendor)

        f.set_renderer('store', self.render_corepos_store)

        f.set_type('notes', 'text')

    def core_office_object_url(self, office_url, order):
        return '{}/purchasing/ViewPurchaseOrders.php?id={}'.format(
            office_url, order.id)

    def get_row_data(self, order):
        return self.Session.query(corepos.PurchaseOrderItem)\
                           .filter(corepos.PurchaseOrderItem.order == order)

    def get_parent(self, item):
        return item.order

    def configure_row_grid(self, g):
        super(PurchaseOrderView, self).configure_row_grid(g)

        g.set_label('received_quantity', "Received Qty.")
        g.filters['received_quantity'].label = "Received Quantity"

        g.set_label('received_total_cost', "Received Cost")
        g.filters['received_total_cost'].label = "Received Total Cost"

        g.set_type('unit_cost', 'currency')
        g.set_type('case_size', 'quantity')
        g.set_type('quantity', 'quantity')
        g.set_type('received_quantity', 'quantity')
        g.set_type('received_total_cost', 'currency')


def defaults(config, **kwargs):
    base = globals()

    PurchaseOrderView = kwargs.get('PurchaseOrderView', base['PurchaseOrderView'])
    PurchaseOrderView.defaults(config)


def includeme(config):
    defaults(config)



# -*- coding: utf-8; -*-
################################################################################
#
#  Rattail -- Retail Software Framework
#  Copyright © 2010-2023 Lance Edgar
#
#  This file is part of Rattail.
#
#  Rattail is free software: you can redistribute it and/or modify it under the
#  terms of the GNU General Public License as published by the Free Software
#  Foundation, either version 3 of the License, or (at your option) any later
#  version.
#
#  Rattail is distributed in the hope that it will be useful, but WITHOUT ANY
#  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
#  details.
#
#  You should have received a copy of the GNU General Public License along with
#  Rattail.  If not, see <http://www.gnu.org/licenses/>.
#
################################################################################
"""
Purchase views
"""

import csv
import os

from tailbone.views.purchases import core as base


class PurchaseView(base.PurchaseView):
    """
    Expose some extra features per CORE-POS integration.
    """

    def template_kwargs_view(self, **kwargs):
        kwargs = super().template_kwargs_view(**kwargs)
        app = self.get_rattail_app()
        corepos = app.get_corepos_handler()

        url = corepos.get_office_url()
        if url:
            url = f'{url}/purchasing/ImportPurchaseOrder.php'
        kwargs['corepos_import_url'] = url

        return kwargs

    def download_for_corepos(self):
        """
        View for downloading a Purchase data file for import to CORE-POS.
        """
        purchase = self.get_instance()
        app = self.get_rattail_app()
        tmpdir = app.make_temp_dir()
        path = os.path.join(tmpdir, 'Purchase {}.csv'.format(purchase.id_str))

        fields = [
            'sku',
            'cost_total',
            'quantity_units',
            'quantity_cases',
            'units_per_case',
            'unit_size',
            'brand',
            'description',
            'upc_without_check',
            'upc_with_check',
        ]

        rows = []

        def collect(item, i):
            upc = item.upc
            if upc:
                # TODO: i am still confused by what really is expected for the
                # 'Unit Size' field here.  CORE *behavior* seems to prefer what
                # logically maps to Rattail `Product.unit_size` (e.g. 12) but
                # the data type for CORE `PurchaseOrderItems.unitSize` is varchar
                # and code comments imply e.g. '12 OZ' would also be expected.
                # for now i am being "conservative" and mimicking CORE *behavior*.
                unit_size = None
                if item.product:
                    unit_size = app.render_quantity(item.product.unit_size)
                rows.append({
                    'sku': item.vendor_code,
                    'cost_total': item.po_total,
                    'quantity_units': app.render_quantity(item.units_ordered),
                    'quantity_cases': app.render_quantity(item.cases_ordered),
                    'units_per_case': app.render_quantity(item.case_quantity),
                    'unit_size': unit_size,
                    'brand': item.brand_name,
                    'description': item.description,
                    'upc_without_check': str(upc)[:-1],
                    'upc_with_check': str(upc),
                })

        self.progress_loop(collect, purchase.items, None, # TODO
                           message="Converting data to CSV")

        with open(path, 'w') as f:
            writer = csv.DictWriter(f, fields)
            # note, the CORE importer does not really need or expect a header
            # TODO: seems like it should? b/c would be more helpful for humans
            #writer.writeheader()
            writer.writerows(rows)

        return self.file_response(path)

    @classmethod
    def defaults(cls, config):
        cls._corepos_purchase_defaults(config)
        cls._purchase_defaults(config)
        cls._defaults(config)

    @classmethod
    def _corepos_purchase_defaults(cls, config):
        route_prefix = cls.get_route_prefix()
        instance_url_prefix = cls.get_instance_url_prefix()
        permission_prefix = cls.get_permission_prefix()
        model_title = cls.get_model_title()

        # download for core-pos
        config.add_route('{}.download_for_corepos'.format(route_prefix), '{}/download-for-corepos'.format(instance_url_prefix))
        config.add_view(cls, attr='download_for_corepos', route_name='{}.download_for_corepos'.format(route_prefix),
                        permission='{}.download_for_corepos'.format(permission_prefix))
        config.add_tailbone_permission(permission_prefix, '{}.download_for_corepos'.format(permission_prefix),
                                       "Download {} for import to CORE-POS".format(model_title))


def includeme(config):
    base.defaults(config, **{'PurchaseView': PurchaseView})
    config.include('tailbone.views.purchases.credits')



# -*- coding: utf-8; -*-
################################################################################
#
#  Rattail -- Retail Software Framework
#  Copyright © 2010-2023 Lance Edgar
#
#  This file is part of Rattail.
#
#  Rattail is free software: you can redistribute it and/or modify it under the
#  terms of the GNU General Public License as published by the Free Software
#  Foundation, either version 3 of the License, or (at your option) any later
#  version.
#
#  Rattail is distributed in the hope that it will be useful, but WITHOUT ANY
#  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
#  details.
#
#  You should have received a copy of the GNU General Public License along with
#  Rattail.  If not, see <http://www.gnu.org/licenses/>.
#
################################################################################
"""
CORE-POS transaction views (base)
"""

from tailbone_corepos.views.corepos.master import CoreMasterView


class TransactionDetailMasterView(CoreMasterView):
    """
    Master view for "current" transaction details.
    """
    labels = {
        'store_row_id': "Store Row ID",
        'store_id': "Store ID",
        'pos_row_id': "POS Row ID",
        'transaction_id': "Transaction ID",
        'upc': "UPC",
        'tax_rate_id': "Tax Rate ID",
    }

    grid_columns = [
        'date_time',
        'register_number',
        'transaction_number',
        'card_number',
        'store_row_id',
        'upc',
        'description',
        'quantity',
        'unit_price',
        'discount',
        'total',
    ]

    def configure_grid(self, g):
        super().configure_grid(g)
        app = self.get_rattail_app()

        g.filters['date_time'].default_active = True
        g.filters['date_time'].default_verb = 'equal'
        g.filters['date_time'].default_value = str(app.today())

        g.set_type('quantity', 'quantity')
        g.set_type('item_quantity', 'quantity')

        g.set_type('cost', 'currency')
        g.set_type('unit_price', 'currency')
        g.set_type('reg_price', 'currency')
        g.set_type('discount', 'currency')
        g.set_type('member_discount', 'currency')
        g.set_type('volume_special', 'currency')
        g.set_type('total', 'currency')

        g.set_sort_defaults('date_time', 'desc')

        g.set_label('register_number', "Register")
        g.set_label('transaction_number', "Trans. No.")
        g.set_label('card_number', "Card No.")
        g.set_label('department_number', "Dept. No.")

        g.set_link('upc')
        g.set_link('description')

    def configure_form(self, f):
        super().configure_form(f)

        f.set_type('quantity', 'quantity')
        f.set_type('item_quantity', 'quantity')

        f.set_type('cost', 'currency')
        f.set_type('unit_price', 'currency')
        f.set_type('reg_price', 'currency')
        f.set_type('discount', 'currency')
        f.set_type('member_discount', 'currency')
        f.set_type('volume_special', 'currency')
        f.set_type('total', 'currency')

    def get_xref_buttons(self, detail):
        app = self.get_rattail_app()
        corepos = app.get_corepos_handler()
        url = corepos.get_office_url()
        if url:

            dt = detail.date_time
            txnkey = f'{detail.employee_number}-{detail.register_number}-{detail.transaction_number}'
            rendered_url = f'{url}/admin/LookupReceipt/RenderReceiptPage.php?year={dt.year}&month={dt.month}&day={dt.day}&receipt={txnkey}'
            raw_url = f'{url}/admin/LookupReceipt/RawReceipt.php?date={dt.date()}&trans={txnkey}'

            return [
                self.make_xref_button(url=rendered_url,
                                      text="View in CORE Office (rendered)"),
                self.make_xref_button(url=raw_url,
                                      text="View in CORE Office (raw)"),
            ]



# -*- coding: utf-8; -*-
################################################################################
#
#  Rattail -- Retail Software Framework
#  Copyright © 2010-2023 Lance Edgar
#
#  This file is part of Rattail.
#
#  Rattail is free software: you can redistribute it and/or modify it under the
#  terms of the GNU General Public License as published by the Free Software
#  Foundation, either version 3 of the License, or (at your option) any later
#  version.
#
#  Rattail is distributed in the hope that it will be useful, but WITHOUT ANY
#  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
#  details.
#
#  You should have received a copy of the GNU General Public License along with
#  Rattail.  If not, see <http://www.gnu.org/licenses/>.
#
################################################################################
"""
CORE-POS group views
"""

from collections import OrderedDict

from corepos.db.office_op import model as corepos

from webhelpers2.html import HTML

from .master import CoreOfficeMasterView


class UserGroupView(CoreOfficeMasterView):
    """
    Master view for CORE Office groups
    """
    model_class = corepos.UserGroup
    model_key = 'group_id'
    model_title = "CORE-POS User Group"
    url_prefix = '/core-pos/user-groups'
    route_prefix = 'corepos.user_groups'
    filterable = False
    pageable = False

    # TODO: maybe add support for these?
    creatable = False
    editable = False
    deletable = False

    labels = {
        'group_id': "Group ID",
    }

    grid_columns = [
        'group_id',
        'name',
    ]

    form_fields = [
        'group_id',
        'name',
        'users',
    ]

    def get_data(self, session=None):
        groups = OrderedDict()

        if session is None:
            session = self.Session()
        query = session.query(corepos.UserGroup)\
                       .order_by(corepos.UserGroup.group_id,
                                 corepos.UserGroup.name)

        for usergroup in query:
            if usergroup.group_id not in groups:
                groups[usergroup.group_id] = self.normalize(usergroup)

        return list(groups.values())

    def normalize(self, group):
        return {
            'group_id': group.group_id,
            'name': group.name,
        }

    def configure_grid(self, g):
        super(UserGroupView, self).configure_grid(g)

        g.sorters['group_id'] = g.make_simple_sorter('group_id')
        g.sorters['name'] = g.make_simple_sorter('name', foldcase=True)

        g.set_sort_defaults('group_id')

        g.set_link('group_id')
        g.set_link('name')

    def get_instance(self):
        gid = self.request.matchdict['group_id']
        group = self.Session.query(corepos.UserGroup)\
                            .filter(corepos.UserGroup.group_id == gid)\
                            .first()
        return self.normalize(group)

    def get_instance_title(self, group):
        return group['name']

    def configure_form(self, f):
        super(UserGroupView, self).configure_form(f)

        if self.creating or self.editing:
            f.remove('users')
        else:
            f.set_renderer('users', self.render_users)

    def render_users(self, group, field):
        groups = self.Session.query(corepos.UserGroup)\
                             .filter(corepos.UserGroup.group_id == group['group_id'])\
                             .order_by(corepos.UserGroup.username)\
                             .all()

        route_prefix = self.get_route_prefix()
        permission_prefix = self.get_permission_prefix()
        factory = self.get_grid_factory()
        g = factory(
            key='{}.users'.format(route_prefix),
            data=[],
            columns=[
                'username',
            ],
        )

        return HTML.literal(
            g.render_buefy_table_element(data_prop='usersData'))

    def template_kwargs_view(self, **kwargs):
        group_info = kwargs['instance']

        users_data = []
        usergroups = self.Session.query(corepos.UserGroup)\
                                 .filter(corepos.UserGroup.group_id == group_info['group_id'])\
                                 .order_by(corepos.UserGroup.username)
        for usergroup in usergroups:
            users_data.append({'username': usergroup.username})
        kwargs['users_data'] = users_data

        return kwargs


def defaults(config, **kwargs):
    base = globals()

    UserGroupView = kwargs.get('UserGroupView', base['UserGroupView'])
    UserGroupView.defaults(config)


def includeme(config):
    defaults(config)



# -*- coding: utf-8; -*-
################################################################################
#
#  Rattail -- Retail Software Framework
#  Copyright © 2010-2023 Lance Edgar
#
#  This file is part of Rattail.
#
#  Rattail is free software: you can redistribute it and/or modify it under the
#  terms of the GNU General Public License as published by the Free Software
#  Foundation, either version 3 of the License, or (at your option) any later
#  version.
#
#  Rattail is distributed in the hope that it will be useful, but WITHOUT ANY
#  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
#  details.
#
#  You should have received a copy of the GNU General Public License along with
#  Rattail.  If not, see <http://www.gnu.org/licenses/>.
#
################################################################################
"""
CORE POS batch views
"""

from corepos.db.office_op import model as corepos
from corepos import enum as corepos_enum

from .master import CoreOfficeMasterView


class BatchTypeView(CoreOfficeMasterView):
    """
    Master view for batch types
    """
    model_class = corepos.BatchType
    model_title = "CORE-POS Batch Type"
    url_prefix = '/core-pos/batch-types'
    route_prefix = 'corepos.batch_types'

    labels = {
        'editor_ui': "Editor UI",
    }

    def configure_grid(self, g):
        super(BatchTypeView, self).configure_grid(g)

        g.set_enum('discount_type', corepos_enum.BATCH_DISCOUNT_TYPE)

        g.set_sort_defaults('id')

        g.set_link('id')
        g.set_link('description')

    def configure_form(self, f):
        super(BatchTypeView, self).configure_form(f)

        f.set_enum('discount_type', corepos_enum.BATCH_DISCOUNT_TYPE)


class BatchView(CoreOfficeMasterView):
    """
    Master view for batches
    """
    model_class = corepos.Batch
    model_title = "CORE-POS Batch"
    model_title_plural = "CORE-POS Batches"
    url_prefix = '/core-pos/batches'
    route_prefix = 'corepos.batches'

    has_rows = True
    model_row_class = corepos.BatchItem
    rows_viewable = False
    rows_downloadable_xlsx = True

    grid_columns = [
        'id',
        'name',
        'batch_type',
        'item_count',
        'start_date',
        'end_date',
    ]

    form_fields = [
        'id',
        'name',
        'batch_type',
        'discount_type',
        'priority',
        'start_date',
        'end_date',
        'owner',
        'trans_limit',
        'notes',
    ]

    row_labels = {
        'id': "ID",
        'batch_id': "Batch ID",
        'upc': "UPC",
    }

    row_grid_columns = [
        'id',
        'upc',
        'sale_price',
        'group_sale_price',
        'active',
        'price_method',
        'quantity',
        'sign_multiplier',
        'cost',
    ]

    def configure_grid(self, g):
        super(BatchView, self).configure_grid(g)

        g.set_renderer('start_date', self.render_local_date)
        g.set_renderer('end_date', self.render_local_date)

        g.set_renderer('item_count', self.render_item_count)

        g.set_sort_defaults('id', 'desc')

        g.set_link('id')
        g.set_link('name')

    def render_item_count(self, batch, field):
        return str(len(batch.items))

    def configure_form(self, f):
        super(BatchView, self).configure_form(f)

        f.set_type('start_date', 'datetime_local')
        f.set_type('end_date', 'datetime_local')

    def get_row_data(self, batch):
        return self.Session.query(corepos.BatchItem)\
                           .filter(corepos.BatchItem.batch == batch)

    def get_parent(self, item):
        return item.batch

    def configure_row_grid(self, g):
        super(BatchView, self).configure_row_grid(g)

        g.set_type('sale_price', 'currency')
        g.set_type('group_sale_price', 'currency')
        # TODO: some older installs don't have this field
        if hasattr(corepos.BatchItem, 'cost'):
            g.set_type('cost', 'currency')
        else:
            g.remove('cost')

        g.set_sort_defaults('id')


def defaults(config, **kwargs):
    base = globals()

    BatchTypeView = kwargs.get('BatchTypeView', base['BatchTypeView'])
    BatchTypeView.defaults(config)

    BatchView = kwargs.get('BatchView', base['BatchView'])
    BatchView.defaults(config)


def includeme(config):
    defaults(config)



# -*- coding: utf-8; -*-
################################################################################
#
#  Rattail -- Retail Software Framework
#  Copyright © 2010-2023 Lance Edgar
#
#  This file is part of Rattail.
#
#  Rattail is free software: you can redistribute it and/or modify it under the
#  terms of the GNU General Public License as published by the Free Software
#  Foundation, either version 3 of the License, or (at your option) any later
#  version.
#
#  Rattail is distributed in the hope that it will be useful, but WITHOUT ANY
#  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
#  details.
#
#  You should have received a copy of the GNU General Public License along with
#  Rattail.  If not, see <http://www.gnu.org/licenses/>.
#
################################################################################
"""
Person views
"""

from collections import OrderedDict

from tailbone.views import ViewSupplement


class PersonViewSupplement(ViewSupplement):
    """
    Person view supplement for CORE integration
    """
    route_prefix = 'people'

    labels = {
        'corepos_customer_id': "CORE-POS Customer ID",
    }

    def get_grid_query(self, query):
        model = self.model
        return query.outerjoin(model.CorePerson)

    def configure_grid(self, g):
        model = self.model
        g.set_filter('corepos_customer_id', model.CorePerson.corepos_customer_id)

    def configure_form(self, f):
        if not self.master.creating:
            f.append('corepos_customer_id')
            f.set_required('corepos_customer_id', False)

    def get_version_child_classes(self):
        model = self.model
        return [model.CorePerson]

    def get_context_for_customer(self, customer, context):

        if customer.corepos_card_number:
            app = self.get_rattail_app()
            corepos = app.get_corepos_handler()
            url = corepos.get_office_member_url(customer.corepos_card_number)
            if url:
                context['external_links'].append({'label': "View in CORE Office",
                                                  'url': url})

        return context

    def get_member_xref_buttons(self, person):
        buttons = OrderedDict()
        app = self.get_rattail_app()
        corepos = app.get_corepos_handler()
        office_url = corepos.get_office_url()
        if office_url:
            kw = {'office_url': office_url}

            for member in person.members:
                url = corepos.get_office_member_url(member.number, **kw)
                buttons[member.uuid] = {'url': url, 'text': "View in CORE Office"}

            for customer in person.customers:
                for member in customer.members:
                    if member.uuid not in buttons:
                        url = corepos.get_office_member_url(member.number, **kw)
                        buttons[member.uuid] = {'url': url, 'text': "View in CORE Office"}

        return buttons.values()


def includeme(config):
    PersonViewSupplement.defaults(config)
